<?php

/**
 * @file
 */

function sudo_settings($form_state) {
  if (is_array($form_state['values']['sudo_remove']) || is_array($form_state['values']['sudo_role_remove'])) {
    return sudo_multiple_delete_confirm($form_state, $form_state['values']['sudo_remove'], $form_state['values']['sudo_role_remove']);
  }
  $form['help'] = array(
    '#value' => '<h2>' . t('Instructions') . '</h2><p>' .
                  t('Sudo module allows administrative users
                  to use the site in the same way that a normal user would, but
                  "sudo" as needed to obtain extra permissions for performing administrative tasks.'
                  . '</p><p>' .
                  'To use this module, you must specify one or more sudoers.  Those
                  users will then have access to a button on every page which will add or remove the site\'s
                  administrative roles on their account.  The roles to be added and removed can be configured on a
                  per-user basis with the form below.')
                  . '</p><p>' .
                  'You may wish to add one or more roles which will cause any
                  users saved with those roles to be automatically added to the
                  sudoers; you can do that under "Automatic Sudo Roles" below.' . '</p>',
  );

  $q = db_query("SELECT * FROM {role}");
  while ($r = db_fetch_object($q)) {
    $roles[$r->rid] = check_plain($r->name);
  }
  $form['default_roles_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Automatic Sudo Roles'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('When users are saved with one or more of these roles,
      they will automatically be added as a sudo user with the corresponding
      roles as sudo roles.'),
    'sudo_default_roles' => array(
      '#type' => 'checkboxes',
      '#title' => '',
      '#options' => $roles,
      '#default_value' => variable_get('sudo_default_roles', array()),
    ),
  );

  // User list
  $form['sudo_users'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sudo Users'),
    '#description' => t('Add users to this list using the form above.  After users are added,
                        specify a set of roles to be added or removed for each user when sudoing.'),
  );

  if (db_result(db_query("SELECT uid FROM {sudo}"))) {
    
    $q = db_query("SELECT s.*, u.name FROM {sudo} s LEFT JOIN {users} u USING (uid)");
    while ($u = db_fetch_object($q)) {
      $sudo_roles = $u->roles ? unserialize($u->roles) : array();
      $form['sudo_users']['checkboxes'][$u->uid]['roles'] = array('#type' => 'checkboxes', '#options' => $roles, '#default_value' => $sudo_roles);
      $form['sudo_users']['checkboxes'][$u->uid]['name'] = array('#value' => check_plain($u->name));
      $form['sudo_users']['checkboxes'][$u->uid]['remove'] = array('#type' => 'checkbox', '#default_value' => FALSE);
      $form['sudo_users']['checkboxes'][$u->uid]['#tree'] = TRUE;
      $form['sudo_users']['checkboxes']['roles'] = array('#value' => $roles, '#tree' => TRUE);
      $form['sudo_users']['checkboxes']['#theme'] = 'sudo_checkboxes';
      $form['sudo_users']['checkboxes']['#tree'] = TRUE;
    }

  }

  // Add user
  $form['new_sudoer'] = array(
    '#type' => 'textfield',
    '#title' => t('Add User'),
    '#description' => t('Type in a uid or part of a username.'),
    '#autocomplete_path' => 'user/autocomplete',
  );

  // Sudo settings
  $form['button'] = array(
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#title' => t('Button Settings'),
  );
  $form['button']['sudo_title_sudoing'] = array(
    '#type' => 'textfield',
    '#title' => t('Title when sudoing'),
    '#default_value' => variable_get('sudo_title_sudoing', 'sudo'),
  );
  $form['button']['sudo_title_normal'] = array(
    '#type' => 'textfield',
    '#title' => t('Title when normal'),
    '#default_value' => variable_get('sudo_title_normal', 'sudo'),
  );
  $form['button']['sudo_button_path'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use Theme Path'),
    '#default_value' => variable_get('sudo_button_path', FALSE),
    '#description' => t('If you select this option, you may either copy the sudo_button folder to your theme directory and make changes as you wish, or implement your own css.'),
  );
  $form['button']['sudo_hide_button'] = array(
    '#type' => 'checkbox',
    '#title' => t('Hide Sudo Button'),
    '#default_value' => variable_get('sudo_hide_button', FALSE),
    '#description' => t('Check this box to disable display of the standard button.  You will have to implement the button in your own theme or in a php block.'),
  );

  // Submit
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  drupal_add_css(drupal_get_path('module', 'sudo') . '/sudo_settings.css');
  return $form;
}

/**
 * Theme the administer permissions page.
 *
 * @ingroup themeable
 */
function theme_sudo_checkboxes(&$form) {

  $header[] = t('Users');
  foreach ($form['roles']['#value'] as $rid => $role_name) {
    $header[] = array('data' => $role_name, 'class' => 'centered');
  }
  $header[] = array('data' => t('Remove'), 'class' => 'centered remove');
  unset($form['roles']);

  $rows = array();

  foreach (element_children($form) as $uid) {
    // Don't take form control structures
    if (is_numeric($uid)) {
      $row = array();
      $row[] = array('data' => $form[$uid]['name']['#value']);
      // Module name
      foreach (element_children($form[$uid]['roles']) as $rid) {
        if (is_numeric($rid)) {
          $title = $form[$uid]['name']['#value'] . ' : ' . $form[$uid][$rid]['#title'];
          $form[$uid]['roles'][$rid]['#title'] = '';
          $row[] = array('data' => drupal_render($form[$uid]['roles'][$rid]), 'class' => 'checkbox', 'title' => $title);
        }
      }
      $row[] = array('data' => drupal_render($form[$uid]['remove']), 'class' => 'checkbox centered remove', 'title' => $form[$uid]['name']['#value'] . ' : ' . t('Remove'));
      unset($form[$uid]['name']);
      $rows[] = $row;
    }
  }
  $output = theme('table', $header, $rows, array('id' => 'permissions'));
  $output .= drupal_render($form);
  return $output;
}

function sudo_settings_validate($form, &$form_state) {
  if ($form_state['values']['new_sudoer'] && !db_result(db_query("SELECT uid FROM {users} WHERE name = '%s'", $form_state['values']['new_sudoer']))) {
    form_set_error('new_sudoer', t('Not a valid username'));
  }
}

function sudo_settings_submit($form, &$form_state) {
  if ($uid = db_result(db_query("SELECT uid FROM {users} WHERE name = '%s'", $form_state['values']['new_sudoer']))) { // intentional assignment of $uid
    if (!db_result(db_query("SELECT uid FROM {sudo} WHERE uid = %d", $uid))) {
      db_query("INSERT INTO {sudo} (uid) VALUES (%d)", $uid);
    }
  }
  if (is_array($form_state['values']['checkboxes'])) {
    foreach ($form_state['values']['checkboxes'] as $uid => $values) {
      if ( is_array($form_state['values']['checkboxes'][$uid]) && is_numeric($uid) ) {
        $save_roles = array_values(array_filter($values['roles']));
        $current_roles = sudo_roles($uid);

        // add all users scheduled for removal to the sudo_remove value, with checkboxes for all sudo roles
        if ($values['remove']) {
          $form_state['rebuild'] = TRUE;
          $form_state['values']['sudo_remove'][$uid] = $current_roles;
        }
        elseif ($diff = array_diff($current_roles, $save_roles)) { // intentional assignment of $diff
          $form_state['rebuild'] = TRUE;
          $form_state['values']['sudo_role_remove'][$uid] = $diff;
          db_query("UPDATE {sudo} SET roles = '%s' WHERE uid = %d", serialize(array_merge($save_roles, $current_roles)), $uid);
        }
        else {
          db_query("UPDATE {sudo} SET roles = '%s' WHERE uid = %d", serialize($save_roles), $uid);
        }
      }
    }
  }
  foreach (array('sudo_hide_button', 'sudo_title_sudoing', 'sudo_title_normal', 'sudo_button_path') as $field) {
    variable_set($field, $form_state['values'][$field]);
  }
  variable_set('sudo_default_roles', array_keys(array_filter($form_state['values']['sudo_default_roles'])));
}

function sudo_multiple_delete_confirm(&$form_state, $sudo_remove, $sudo_role_remove) {
  $roles = user_roles(TRUE);
  foreach (array('sudo_remove', 'sudo_role_remove') as $key) {
    if (is_array($$key)) {
      $form[$key] = array(
        '#type' => 'fieldset',
        '#title' => $key == 'sudo_remove' ? t('Users to be removed') : t('Roles to be removed'),
        '#description' => $key == 'sudo_remove' ?
          t('The following users will be <b>removed from sudoers.</b><br/>
            Any roles checked below will be permanently added to their accounts.<br/>
            <b>All un-checked roles will be removed from their accounts.</b>') :
          t('The following users will remain as sudoers, but with <b>fewer permissions</b> when sudoing.<br/>
            Any roles checked below will be permanently added to their accounts.<br/>
            <b>All un-checked roles will be removed from their accounts.</b>'),
        '#tree' => TRUE,
      );
      foreach ($$key as $uid => $user_roles) {
        $options = array();
        foreach ($user_roles as $rid) {
          $options[$rid] = check_plain($roles[$rid]);
        }
        $form[$key][$uid] = array(
          '#type' => 'checkboxes',
          '#title' => check_plain(db_result(db_query("SELECT name FROM {users} WHERE uid = %d", $uid))),
          '#options' => $options,
        );
      }
    }
  }
  $form['#submit'][] = 'sudo_multiple_delete_confirm_submit';
  return confirm_form($form,
          t('Which roles should these users have?'), 'admin/user/sudo',
          t('Your changes include removing sudo roles from some users.<br/>Please check
            any roles that the users should continue to have on a permanent basis.<br/>
            <b>All un-checked roles will be removed from users accounts.</b>'),
          t('Continue'), t('Cancel')
        );
}

function sudo_multiple_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    foreach (array('sudo_remove', 'sudo_role_remove') as $key) {
      if (is_array($form_state['values'][$key])) {
        foreach ($form_state['values'][$key] as $uid => $remove_roles) {
          $keep_roles = array_filter($remove_roles);
          $remove_roles = array_keys($remove_roles);
          $current_roles = sudo_roles($uid);

          // save the new sudo roles
          if ($key == 'sudo_remove') {
            db_query("DELETE FROM {sudo} WHERE uid = %d", $uid);
          }
          else {
            db_query("UPDATE {sudo} SET roles = '%s' WHERE uid = %d", serialize(array_diff($current_roles, $remove_roles)), $uid);
          }

          // save the user account
          if (is_array($remove_roles) && !empty($remove_roles)) { // sanity check
            $placeholders = db_placeholders($remove_roles, 'int');
            array_unshift($remove_roles, $uid);
            db_query("DELETE FROM {users_roles} WHERE uid = %d AND rid IN ($placeholders)", $remove_roles);
            foreach ($keep_roles as $rid) {
              db_query("INSERT INTO {users_roles} (uid, rid) VALUES (%d, %d)", $uid, $rid);
            }
          }

        }
      }
    }
  }
}

function sudo_switch() {
  $title = 'sudo_title_' . ($state = $_SESSION['sudoing'] ? 'sudoing' : 'normal');
  $path = variable_get('sudo_button_path', drupal_get_path('module', 'sudo'));
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t(check_plain(variable_get($title, 'sudo'))),
  );
  return $form;
}

function sudo_switch_submit($form, &$form_state) {
  if (variable_get('site_offline', FALSE)) {
    drupal_set_message(t("You can't un-sudo when the site is in maintenance mode."), 'error');
    return;
  }
  global $user;
  if ($sudo_roles = sudo_roles()) { // intentional assignment of $sudo_roles

    // if user is missing sudo roles, add them
    $placeholders = db_placeholders($sudo_roles);
    array_unshift($sudo_roles, $user->uid);
    db_query("DELETE FROM {users_roles} WHERE uid = %d AND rid IN ($placeholders)", $sudo_roles);
    if (!$_SESSION['sudoing']) {
      array_shift($sudo_roles);
      foreach ($sudo_roles as $rid) {
        db_query("INSERT INTO {users_roles} VALUES (%d, %d)", $user->uid, $rid);
      }
    }

  }
}

/**
 * Returns a themed form for switching to and from sudo roles.
 * @param string $state either "sudoing" or "normal", used for display of
 * pictures surrounding the form's button
 * @param string $path the path to the themable elements and css files; by default,
 * either the module's directory or the theme's directory, depending on configuration
 * @return string html of a form with only a button
 */
function theme_sudo_button($state) {
  $form = drupal_get_form('sudo_switch');
  $output .= "<div id='sudo-button' class='$state'>";
  $output .= $form;
  $output .= "</div>";
  return $output;
}

/**
 * This function ensures that sudoers who are not sudoing when the site is taken
 * offline will not be locked out; instead, all their sudo roles will be added
 * to their account at that time.
 * @param array $form
 * @param array $form_state
 */
function sudo_site_offline_submit($form, &$form_state) {
  if ($form_state['values']['site_offline']) {
    $q = db_query("SELECT * FROM {sudo}");
    while ($r = db_fetch_object($q)) {
      $roles = $r->roles ? unserialize($r->roles) : FALSE;
      if (is_array($roles) && count($roles)) {
        array_unshift($roles, $r->uid);
        $placeholders = db_placeholders($roles, 'int');
        db_query("DELETE FROM {users_roles} WHERE uid = %d AND rid IN ($placeholders)", $roles);
        array_shift($roles);
        foreach ($roles as $rid) {
          db_query("INSERT INTO {users_roles} (uid, rid) VALUES (%d, %d)", $r->uid, $rid);
        }
      }
    }
  }
}
